/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Communicator client code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#ifndef nsSingleByteCharSetProber_generated_h__
#define nsSingleByteCharSetProber_generated_h__

#define NUM_OF_SEQUENCE_MODELS 120

extern const SequenceModel Iso_8859_6ArabicModel;
extern const SequenceModel Windows_1256ArabicModel;

extern const SequenceModel Windows_1251BelarusianModel;
extern const SequenceModel Iso_8859_5BelarusianModel;

extern const SequenceModel Windows_1251BulgarianModel;
extern const SequenceModel Iso_8859_5BulgarianModel;

extern const SequenceModel Iso_8859_1CatalanModel;
extern const SequenceModel Windows_1252CatalanModel;

extern const SequenceModel Iso_8859_2CzechModel;
extern const SequenceModel Windows_1250CzechModel;
extern const SequenceModel Ibm852CzechModel;
extern const SequenceModel Mac_CentraleuropeCzechModel;

extern const SequenceModel Iso_8859_15DanishModel;
extern const SequenceModel Iso_8859_1DanishModel;
extern const SequenceModel Windows_1252DanishModel;
extern const SequenceModel Ibm865DanishModel;

extern const SequenceModel Iso_8859_1GermanModel;
extern const SequenceModel Windows_1252GermanModel;

extern const SequenceModel Iso_8859_7GreekModel;
extern const SequenceModel Windows_1253GreekModel;
extern const SequenceModel Cp737GreekModel;

extern const SequenceModel Iso_8859_1EnglishModel;
extern const SequenceModel Windows_1252EnglishModel;

extern const SequenceModel Iso_8859_3EsperantoModel;

extern const SequenceModel Iso_8859_15SpanishModel;
extern const SequenceModel Iso_8859_1SpanishModel;
extern const SequenceModel Windows_1252SpanishModel;

extern const SequenceModel Iso_8859_4EstonianModel;
extern const SequenceModel Iso_8859_13EstonianModel;
extern const SequenceModel Iso_8859_15EstonianModel;
extern const SequenceModel Windows_1252EstonianModel;
extern const SequenceModel Windows_1257EstonianModel;

extern const SequenceModel Iso_8859_1FinnishModel;
extern const SequenceModel Iso_8859_4FinnishModel;
extern const SequenceModel Iso_8859_9FinnishModel;
extern const SequenceModel Iso_8859_13FinnishModel;
extern const SequenceModel Iso_8859_15FinnishModel;
extern const SequenceModel Windows_1252FinnishModel;

extern const SequenceModel Iso_8859_15FrenchModel;
extern const SequenceModel Iso_8859_1FrenchModel;
extern const SequenceModel Windows_1252FrenchModel;

extern const SequenceModel Iso_8859_15IrishModel;
extern const SequenceModel Iso_8859_1IrishModel;
extern const SequenceModel Iso_8859_9IrishModel;
extern const SequenceModel Windows_1252IrishModel;

extern const SequenceModel Iso_8859_8HebrewModel;
extern const SequenceModel Windows_1255HebrewModel;
extern const SequenceModel Ibm862HebrewModel;


extern const SequenceModel Iso_8859_2CroatianModel;
extern const SequenceModel Iso_8859_13CroatianModel;
extern const SequenceModel Iso_8859_16CroatianModel;
extern const SequenceModel Windows_1250CroatianModel;
extern const SequenceModel Ibm852CroatianModel;
extern const SequenceModel Mac_CentraleuropeCroatianModel;

extern const SequenceModel Iso_8859_2HungarianModel;
extern const SequenceModel Windows_1250HungarianModel;

extern const SequenceModel Iso_8859_1ItalianModel;
extern const SequenceModel Iso_8859_3ItalianModel;
extern const SequenceModel Iso_8859_9ItalianModel;
extern const SequenceModel Iso_8859_15ItalianModel;
extern const SequenceModel Windows_1252ItalianModel;

extern const SequenceModel Georgian_AcademyGeorgianModel;
extern const SequenceModel Georgian_PsGeorgianModel;

extern const SequenceModel Iso_8859_4LithuanianModel;
extern const SequenceModel Iso_8859_10LithuanianModel;
extern const SequenceModel Iso_8859_13LithuanianModel;

extern const SequenceModel Iso_8859_4LatvianModel;
extern const SequenceModel Iso_8859_10LatvianModel;
extern const SequenceModel Iso_8859_13LatvianModel;

extern const SequenceModel Windows_1251MacedonianModel;
extern const SequenceModel Ibm855MacedonianModel;
extern const SequenceModel Iso_8859_5MacedonianModel;

extern const SequenceModel Iso_8859_3MalteseModel;

extern const SequenceModel Ibm865NorwegianModel;
extern const SequenceModel Iso_8859_15NorwegianModel;
extern const SequenceModel Iso_8859_1NorwegianModel;
extern const SequenceModel Windows_1252NorwegianModel;

extern const SequenceModel Iso_8859_2PolishModel;
extern const SequenceModel Iso_8859_13PolishModel;
extern const SequenceModel Iso_8859_16PolishModel;
extern const SequenceModel Windows_1250PolishModel;
extern const SequenceModel Ibm852PolishModel;
extern const SequenceModel Mac_CentraleuropePolishModel;

extern const SequenceModel Iso_8859_15PortugueseModel;
extern const SequenceModel Iso_8859_1PortugueseModel;
extern const SequenceModel Windows_1252PortugueseModel;
extern const SequenceModel Iso_8859_9PortugueseModel;

extern const SequenceModel Iso_8859_2RomanianModel;
extern const SequenceModel Iso_8859_16RomanianModel;
extern const SequenceModel Windows_1250RomanianModel;
extern const SequenceModel Ibm852RomanianModel;

extern const SequenceModel Windows_1251RussianModel;
extern const SequenceModel Iso_8859_5RussianModel;
extern const SequenceModel Koi8_RRussianModel;
extern const SequenceModel Ibm855RussianModel;
extern const SequenceModel Ibm866RussianModel;
extern const SequenceModel Mac_CyrillicRussianModel;

extern const SequenceModel Iso_8859_2SlovakModel;
extern const SequenceModel Windows_1250SlovakModel;
extern const SequenceModel Ibm852SlovakModel;
extern const SequenceModel Mac_CentraleuropeSlovakModel;

extern const SequenceModel Iso_8859_2SloveneModel;
extern const SequenceModel Iso_8859_16SloveneModel;
extern const SequenceModel Windows_1250SloveneModel;
extern const SequenceModel Ibm852SloveneModel;
extern const SequenceModel Mac_CentraleuropeSloveneModel;

extern const SequenceModel Windows_1251SerbianModel;
extern const SequenceModel Iso_8859_5SerbianModel;

extern const SequenceModel Iso_8859_1SwedishModel;
extern const SequenceModel Iso_8859_4SwedishModel;
extern const SequenceModel Iso_8859_9SwedishModel;
extern const SequenceModel Iso_8859_15SwedishModel;
extern const SequenceModel Windows_1252SwedishModel;

extern const SequenceModel Iso_8859_11ThaiModel;
extern const SequenceModel Tis_620ThaiModel;

extern const SequenceModel Iso_8859_3TurkishModel;
extern const SequenceModel Iso_8859_9TurkishModel;

extern const SequenceModel Windows_1251UkrainianModel;

extern const SequenceModel Windows_1258VietnameseModel;
extern const SequenceModel VisciiVietnameseModel;

#endif /* nsSingleByteCharSetProber_generated_h__ */